#!/usr/bin/perl
# ---------------------------------------------------------------- #
#   郵便番号一覧 CSV ファイルをタブ区切りテキストに変換
#   http://www.post.yusei.go.jp/newnumber/lzh/s/ken_all.lzh
#   (c) 2001-2005 Kappe INC. All rights reserved.
# ---------------------------------------------------------------- #
    use strict;
    use Jcode;
    use DB_File;
    use Fcntl;
# ---------------------------------------------------------------- #
    my $CSV_ENC = "sjis";           # CSV ファイルのエンコーディング
    my $DB_ENC = "utf8";            # DB_File中のエンコーディング
    my $DISP_ENC = "euc";           # 表示用のエンコーディング
# ---------------------------------------------------------------- #
    local $| = 1;
    &main( @ARGV );
# ---------------------------------------------------------------- #
sub main {
    my $csvfile = shift;
    my $dbfile = shift;

    # 第3引数でDB文字コードを指定できる
    my $arg3 = shift;
    $DB_ENC = $arg3 if $arg3;

    # DBファイル名を決定する

    if ( $dbfile eq "" ) {
        $dbfile = $csvfile;
        $dbfile =~ s/\.[^\.]+$//;
        $dbfile .= ".db";
    }
    my $dbtemp = $dbfile.".work";
    unlink( $dbtemp ) if ( -f $dbtemp );

    # ファイルを開く

    my $dbinfo = new DB_File::BTREEINFO();
    $dbinfo->{flags} = R_DUP();         # １郵便番号→複数住所に対応
    my $tied;
    my $flag = Fcntl::O_RDWR() | Fcntl::O_CREAT();
    my $dbobj = tie( %$tied, "DB_File",
                $dbtemp,                # ファイル名
                $flag,                  # 作成あり
                0644,                   # パーミション
                $dbinfo );              # B-TREE 形式

    # 変換処理を行う
    my $prev;
    my $c;
    open( CSV, $csvfile ) or die "$! - $csvfile\n";
    while ( my $iline = <CSV> ) {
        last if ( $iline =~ /^\x1a/ );  # EOF

        # UTF-8コードで処理する
        Jcode::convert( \$iline, $DB_ENC, $CSV_ENC );

        # CSVとはいっても「,」の文字は住所には利用されていないので簡易処理
        my @r = split( ",", $iline );
        s/^"(.*)"$/$1/s foreach ( @r );

        # 全角かっこ『（』が入る場合は、フリガナからも半角かっこ『(』を除外
        if ( $r[8] =~ s/（([\000-\177]|[\200-\377]{2})+$//s ) {
            $r[5] =~ s/\([^\(]+$//s;
        }

        #『以下に掲載がない場合』等は削除してしまう
        if ( $r[8] =~ /(^以下に掲載がない場合
                        |の次に番地がくる場合
                        |一円
                        |）
                        |、([\000-\177]|[\200-\377]{2})*
                        )$/xs ) {
            $r[8] = "";
            $r[5] = "";
        }

        # １行をタブ区切りテキストにしてDBファイルに書き込む
        $tied->{$r[2]} = join( "\t", @r );

        # 都道府県が変わったら、画面に都道府県名を表示する
        if ( $prev ne $r[6] ) {
            $prev = $r[6];
            print " $c\n" if $c;
            my $v = $r[6];
            &Jcode::convert( \$v, $DISP_ENC, $DB_ENC );
            printf STDERR ( "%-10s", $v );
            $c = 0;
        }
        print "." if ( $c ++ % 200 == 0 );
    }
    print " $c\n" if ( $c > 1 );

    # ファイル名を置き換える
    rename( $dbtemp, $dbfile );
}
# ---------------------------------------------------------------- #
